@@ -181,7 +181,7 @@ public:
     /// @param values         Destination buffer for the interpolated primvar
     ///                       data
     ///
-    /// @param start          (skip to )index of first value to update
+    /// @param start          index of first value to update
     ///
     /// @param end            Index of last value to update
     ///
@@ -253,36 +253,66 @@ public:
     ///
     /// @param duWeights  Table pointer to the 'u' derivative weights
     ///
-    /// @param dvWeights Table pointer to the 'v' derivative weights
+    /// @param dvWeights  Table pointer to the 'v' derivative weights
+    ///
+    /// @param duuWeights Table pointer to the 'uu' derivative weights
+    ///
+    /// @param duvWeights Table pointer to the 'uv' derivative weights
+    ///
+    /// @param dvvWeights Table pointer to the 'vv' derivative weights
     ///
     LimitStencil( int* size,
                   Index * indices,
                   float * weights,
-                  float * duWeights,
-                  float * dvWeights )
+                  float * duWeights=0,
+                  float * dvWeights=0,
+                  float * duuWeights=0,
+                  float * duvWeights=0,
+                  float * dvvWeights=0)
         : Stencil(size, indices, weights),
           _duWeights(duWeights),
-          _dvWeights(dvWeights) {
+          _dvWeights(dvWeights),
+          _duuWeights(duuWeights),
+          _duvWeights(duvWeights),
+          _dvvWeights(dvvWeights) {
     }
 
-    /// \brief
+    /// \brief Returns the u derivative weights
     float const * GetDuWeights() const {
         return _duWeights;
     }
 
-    /// \brief
+    /// \brief Returns the v derivative weights
     float const * GetDvWeights() const {
         return _dvWeights;
     }
 
+    /// \brief Returns the uu derivative weights
+    float const * GetDuuWeights() const {
+        return _duuWeights;
+    }
+
+    /// \brief Returns the uv derivative weights
+    float const * GetDuvWeights() const {
+        return _duvWeights;
+    }
+
+    /// \brief Returns the vv derivative weights
+    float const * GetDvvWeights() const {
+        return _dvvWeights;
+    }
+
     /// \brief Advance to the next stencil in the table
     void Next() {
        int stride = *_size;
        ++_size;
        _indices += stride;
        _weights += stride;
-       _duWeights += stride;
-       _dvWeights += stride;
+       if (_duWeights) _duWeights += stride;
+       if (_dvWeights) _dvWeights += stride;
+       if (_duuWeights) _duuWeights += stride;
+       if (_duvWeights) _duvWeights += stride;
+       if (_dvvWeights) _dvvWeights += stride;
     }
 
 private:
@@ -291,7 +321,10 @@ private:
     friend class LimitStencilTableFactory;
 
     float * _duWeights,  // pointer to stencil u derivative limit weights
-          * _dvWeights;  // pointer to stencil v derivative limit weights
+          * _dvWeights,  // pointer to stencil v derivative limit weights
+          * _duuWeights, // pointer to stencil uu derivative limit weights
+          * _duvWeights, // pointer to stencil uv derivative limit weights
+          * _dvvWeights; // pointer to stencil vv derivative limit weights
 };
 
 /// \brief Table of limit subdivision stencils.
@@ -305,6 +338,9 @@ class LimitStencilTable : public StencilTable {
                     std::vector<float> const& weights,
                     std::vector<float> const& duWeights,
                     std::vector<float> const& dvWeights,
+                    std::vector<float> const& duuWeights,
+                    std::vector<float> const& duvWeights,
+                    std::vector<float> const& dvvWeights,
                     bool includeCoarseVerts,
                     size_t firstOffset);
 
@@ -326,6 +362,21 @@ public:
         return _dvWeights;
     }
 
+    /// \brief Returns the 'uu' derivative stencil interpolation weights
+    std::vector<float> const & GetDuuWeights() const {
+        return _duuWeights;
+    }
+
+    /// \brief Returns the 'uv' derivative stencil interpolation weights
+    std::vector<float> const & GetDuvWeights() const {
+        return _duvWeights;
+    }
+
+    /// \brief Returns the 'vv' derivative stencil interpolation weights
+    std::vector<float> const & GetDvvWeights() const {
+        return _dvvWeights;
+    }
+
     /// \brief Updates derivative values based on the control values
     ///
     /// \note The destination buffers ('uderivs' & 'vderivs') are assumed to
@@ -339,7 +390,7 @@ public:
     /// @param vderivs        Destination buffer for the interpolated 'v'
     ///                       derivative primvar data
     ///
-    /// @param start          (skip to )index of first value to update
+    /// @param start          index of first value to update
     ///
     /// @param end            Index of last value to update
     ///
@@ -351,6 +402,35 @@ public:
         update(controlValues, vderivs, _dvWeights, start, end);
     }
 
+    /// \brief Updates 2nd derivative values based on the control values
+    ///
+    /// \note The destination buffers ('uuderivs', 'uvderivs', & 'vderivs') are
+    ///       assumed to have allocated at least \c GetNumStencils() elements.
+    ///
+    /// @param controlValues  Buffer with primvar data for the control vertices
+    ///
+    /// @param uuderivs       Destination buffer for the interpolated 'uu'
+    ///                       derivative primvar data
+    ///
+    /// @param uvderivs       Destination buffer for the interpolated 'uv'
+    ///                       derivative primvar data
+    ///
+    /// @param vvderivs       Destination buffer for the interpolated 'vv'
+    ///                       derivative primvar data
+    ///
+    /// @param start          index of first value to update
+    ///
+    /// @param end            Index of last value to update
+    ///
+    template <class T>
+    void Update2ndDerivs(T const *controlValues, T *uuderivs, T *uvderivs, T *vvderivs,
+        int start=-1, int end=-1) const {
+
+        update(controlValues, uuderivs, _duuWeights, start, end);
+        update(controlValues, uvderivs, _duvWeights, start, end);
+        update(controlValues, vvderivs, _dvvWeights, start, end);
+    }
+
     /// \brief Clears the stencils from the table
     void Clear();
 
@@ -361,8 +441,11 @@ private:
     void resize(int nstencils, int nelems);
 
 private:
-    std::vector<float>  _duWeights,  // u derivative limit stencil weights
-                        _dvWeights;  // v derivative limit stencil weights
+    std::vector<float>  _duWeights,   // u  derivative limit stencil weights
+                        _dvWeights,   // v  derivative limit stencil weights
+                        _duuWeights,  // uu derivative limit stencil weights
+                        _duvWeights,  // uv derivative limit stencil weights
+                        _dvvWeights;  // vv derivative limit stencil weights
 };
 
 
@@ -383,7 +466,7 @@ StencilTable::update(T const *controlValues, T *values,
         values += start;
     }
 
-    if (end<start or end<0) {
+    if (end<start || end<0) {
         end = GetNumStencils();
     }
 
@@ -441,7 +524,7 @@ StencilTable::finalize() {
 // Returns a Stencil at index i in the table
 inline Stencil
 StencilTable::GetStencil(Index i) const {
-    assert((not _offsets.empty()) and i<(int)_offsets.size());
+    assert((! _offsets.empty()) && i<(int)_offsets.size());
 
     Index ofs = _offsets[i];
 
@@ -465,15 +548,31 @@ LimitStencilTable::resize(int nstencils, int nelems) {
 // Returns a LimitStencil at index i in the table
 inline LimitStencil
 LimitStencilTable::GetLimitStencil(Index i) const {
-    assert((not GetOffsets().empty()) and i<(int)GetOffsets().size());
+    assert((! GetOffsets().empty()) && i<(int)GetOffsets().size());
 
     Index ofs = GetOffsets()[i];
 
-    return LimitStencil( const_cast<int *>(&GetSizes()[i]),
-                         const_cast<Index *>(&GetControlIndices()[ofs]),
-                         const_cast<float *>(&GetWeights()[ofs]),
-                         const_cast<float *>(&GetDuWeights()[ofs]),
-                         const_cast<float *>(&GetDvWeights()[ofs]) );
+    if (!_duWeights.empty() && !_dvWeights.empty() &&
+        !_duuWeights.empty() && !_duvWeights.empty() && !_dvvWeights.empty()) {
+        return LimitStencil( const_cast<int *>(&GetSizes()[i]),
+                             const_cast<Index *>(&GetControlIndices()[ofs]),
+                             const_cast<float *>(&GetWeights()[ofs]),
+                             const_cast<float *>(&GetDuWeights()[ofs]),
+                             const_cast<float *>(&GetDvWeights()[ofs]),
+                             const_cast<float *>(&GetDuuWeights()[ofs]),
+                             const_cast<float *>(&GetDuvWeights()[ofs]),
+                             const_cast<float *>(&GetDvvWeights()[ofs]) );
+    } else if (!_duWeights.empty() && !_dvWeights.empty()) {
+        return LimitStencil( const_cast<int *>(&GetSizes()[i]),
+                             const_cast<Index *>(&GetControlIndices()[ofs]),
+                             const_cast<float *>(&GetWeights()[ofs]),
+                             const_cast<float *>(&GetDuWeights()[ofs]),
+                             const_cast<float *>(&GetDvWeights()[ofs]) );
+    } else {
+        return LimitStencil( const_cast<int *>(&GetSizes()[i]),
+                             const_cast<Index *>(&GetControlIndices()[ofs]),
+                             const_cast<float *>(&GetWeights()[ofs]) );
+    }
 }
 
 inline LimitStencil
