@@ -6,8 +6,7 @@ var searchData=
   ['patchfacetag',['PatchFaceTag',['../a00607.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Far::PatchTableFactory']]],
   ['patchhandle',['PatchHandle',['../a00595.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Far::PatchTable']]],
   ['patchmap',['PatchMap',['../a00575.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Far']]],
-  ['patchparam',['PatchParam',['../a00587.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Far']]],
-  ['patchparam',['PatchParam',['../a00859.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Osd']]],
+  ['patchparam',['PatchParam',['../a00859.html',1,'PatchParam'],['../a00587.html',1,'PatchParam']]],
   ['patchtable',['PatchTable',['../a00591.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Far']]],
   ['patchtablefactory',['PatchTableFactory',['../a00599.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Far']]],
   ['primvarrefiner',['PrimvarRefiner',['../a00611.html',1,'OpenSubdiv::OPENSUBDIV_VERSION::Far']]],
