@@ -6,8 +6,5 @@ var searchData=
   ['topologylevel_2eh',['topologyLevel.h',['../a00407.html',1,'']]],
   ['topologyrefiner_2eh',['topologyRefiner.h',['../a00410.html',1,'']]],
   ['topologyrefinerfactory_2eh',['topologyRefinerFactory.h',['../a00413.html',1,'']]],
-  ['types_2eh',['types.h',['../a01000.html',1,'']]],
-  ['types_2eh',['types.h',['../a01009.html',1,'']]],
-  ['types_2eh',['types.h',['../a01006.html',1,'']]],
-  ['types_2eh',['types.h',['../a01003.html',1,'']]]
+  ['types_2eh',['types.h',['../a01000.html',1,'(Global Namespace)'],['../a01003.html',1,'(Global Namespace)'],['../a01006.html',1,'(Global Namespace)'],['../a01009.html',1,'(Global Namespace)']]]
 ];
